

-- ============================================================================
-- Best Option Settings (in REAL_TIME) for a Program for a Benchmark:
-- ============================================================================

CREATE VIEW BEST_OPT AS
SELECT	B.BENCHMARK AS BENCH,
	T.INPUT_NO AS INPUT,
	T.OPTIONS, R.MACHINE, R.PROG, R.VER,
	R.AVG_REAL_TIME AS REAL_S
FROM	BENCHMARK B, TESTS T, RUN_STATISTICS R
WHERE	B.BENCHMARK = T.BENCHMARK
AND	T.BENCH_ID = R.BENCH_ID
AND	T.PROG = R.PROG
AND NOT EXISTS(SELECT * FROM TESTS T2, RUN_STATISTICS R2
		WHERE R2.BENCH_ID = T2.BENCH_ID AND R2.PROG = T2.PROG
		AND   T2.BENCHMARK = T.BENCHMARK AND T2.INPUT_NO = T.INPUT_NO
		AND   R2.PROG = R.PROG AND R2.MACHINE = R.MACHINE
		AND   R2.AVG_REAL_TIME < R.AVG_REAL_TIME)
-- Eliminate trivial tests with run time 0: All options are best.
AND EXISTS(SELECT * FROM TESTS T3, RUN_STATISTICS R3
		WHERE R3.BENCH_ID = T3.BENCH_ID AND R3.PROG = T3.PROG
		AND   T3.BENCHMARK = T.BENCHMARK AND T3.INPUT_NO = T.INPUT_NO
		AND   R3.PROG = R.PROG AND R3.MACHINE = R.MACHINE
		AND   R3.AVG_REAL_TIME > 0)
ORDER BY B.SEQ_NO, T.INPUT_NO;

-- ============================================================================
-- Benchmark Results on Machine "bach":
-- ============================================================================

CREATE VIEW BACH_RESULTS AS
SELECT R.BENCH_ID AS BENCH,
	R.PROG AS PROG,
	R.AVG_LOAD_TIME AS LOAD_S,
	R.AVG_EXEC_TIME AS EXEC_S,
	R.AVG_TOTAL_TIME AS TOTAL_S,
	R.AVG_REAL_TIME AS REAL_S,
	R.AVG_MEM AS MEM_MB
FROM   RUN_STATISTICS R, PROG_VER V, TESTS T
WHERE  R.MACHINE = 'bach' AND V.MACHINE = R.MACHINE
AND    R.PROG = V.PROG AND R.VER = V.VER
AND    V.STATUS = 'C'
AND    R.BENCH_ID = T.BENCH_ID AND R.PROG = T.PROG
ORDER BY T.BENCHMARK, T.PROG, T.OPTIONS, T.INPUT_NO, R.BENCH_ID;

-- ============================================================================
-- Benchmark Results on Machine "dbs1":
-- ============================================================================

CREATE VIEW DBS_RESULTS AS
SELECT R.BENCH_ID AS BENCH,
	R.PROG AS PROG,
	R.AVG_LOAD_TIME AS LOAD_S,
	R.AVG_EXEC_TIME AS EXEC_S,
	R.AVG_TOTAL_TIME AS TOTAL_S,
	R.AVG_REAL_TIME AS REAL_S,
	R.AVG_MEM AS MEM_MB
FROM   RUN_STATISTICS R, PROG_VER V, TESTS T
WHERE  R.MACHINE = 'dbs1' AND V.MACHINE = R.MACHINE
AND    R.PROG = V.PROG AND R.VER = V.VER
AND    V.STATUS = 'C'
AND    R.BENCH_ID = T.BENCH_ID AND R.PROG = T.PROG
ORDER BY T.BENCHMARK, T.PROG, T.OPTIONS, T.INPUT_NO, R.BENCH_ID;

