---
title: Benchmarks of Deductive Database Systems, Relational Databases, and Graph Databases
---

We define some problems in a declarative way. We are interested in how different (kinds of) database systems faire in solving the problems. Of course, problem-specific implementations and algorithms are often faster, but we're interested in the declarative programming and specifications-as-programs.

## Authors

* Stefan Brass
* Mario Wenzel

## Problems

A problem has one or more instances that can be solved using the same program or solver. Sub-problems use basically the same solver with some minor adjustment.

### Transitive closure (TC)

The TC problem is generating a Transitive closure of a directed graph. The output
is potentially quadratic in the input size.
The problems are generated using our [graphgen tool](https://gitlab.informatik.uni-halle.de/mwenzel/graphgen) written in Python.

We split the TC problem in the following sub-problems:

* tcff: Calculating the complete transitive closure
* tcbf: Calculating all reachable nodes from a given node

We made the following adjustments for different database systems:

* We added indexes on the base relation wherever appropriate
* For relational databases we just do a recursive query instead of creating a recursive view
* SPARQL (for graph databases) does not support general recursion. We used SPARQL 1.1 property paths to define the problem.

### Problem format matrix

We can generate problem instances in the following formats

|                    | Datalog Facts |  EDN  | Turtle | SQL  | CSV   | TSV   | JSON  |
| ------------------ | :-----------: | :---: | :----: | :--: | :---: | :---: | :---: |
| TC                 | X             |  X    |   X    |  X   | X     | X     | X     |


### Problem Instances in Publications

* WLP18
    * Instances for WLP18 can be generated using [wlp18data](https://gitlab.informatik.uni-halle.de/mwenzel/graphgen/blob/master/wlp18data.sh) from graphgen.


## Systems

Our benchmarking system supports the following systems for at least one problem:

* Prolog Systems
    * [SWI Prolog](http://www.swi-prolog.org/) (from version 7, needs tabling support)
    * [YAP](https://www.dcc.fc.up.pt/~vsc/Yap/) (compiled with tabling support)
    * [XSB](http://xsb.sourceforge.net/)
* Datalog Systems
    * [DLV](http://www.dlvsystem.com/dlv/)
    * [datascript](https://github.com/tonsky/datascript)
* Relational Databases
    * [SQLite](https://www.sqlite.org/) (from Version 3)
* Graph Databases and RDF Stores
    * [Apache Jena](https://jena.apache.org/)

## Benchmarking Matrix

We defined problems 

|             | tcff  | tcbf  |
| ----------- | :---: | :---: |
| SWI         | X     | X     |
| YAP         | X     | X     |
| XSB         | X     | X     |
| DLV         | X     | X     |
| datascript  | X     |       |
| SQLite      | X     |       |
| Jena        | X     |       ||


## Results
