/*=============================================================================
 * Tree structure join operation: Join1 Benchmark with Query a(X,Y).
 *
 * This version tables all derived predicates (with "subsumptive tabling").
 * In this way, duplicates are already eliminated for the intermediate results.
 * This version also uses trie indexes for the database predicates.
 *
 * The main program is in 'bench_lib.P'.
 * This file is a modified version of a file distributed with OpenRuleBench.
 *=============================================================================
 */

/*-----------------------------------------------------------------------------
 * ID of this Benchmark:
 *-----------------------------------------------------------------------------
 */

bench_id('j1axy_allt').


/*-----------------------------------------------------------------------------
 * Options:
 *-----------------------------------------------------------------------------
 */

/* Data load method: */
:- import load_dync/2 from consult.
load_data(Filename) :- load_dync(Filename, a).

/* Tabling method: */
:- use_subsumptive_tabling c1/2.
:- use_subsumptive_tabling b1/2.
:- use_subsumptive_tabling b2/2.
:- use_subsumptive_tabling a/2.

/* Indexes: */
:- index(d2/2, trie).
:- index(c2/2, trie).
:- index(c3/2, trie).
:- index(c4/2, trie).


/*-----------------------------------------------------------------------------
 * Benchmark Program:
 *-----------------------------------------------------------------------------
 */

a(X, Y)  :- b1(X, Z), b2(Z, Y).

b1(X, Y) :- c1(X, Z), c2(Z, Y).
b2(X, Y) :- c3(X, Z), c4(Z, Y).

c1(X, Y) :- d1(X, Z), d2(Z, Y).


/*-----------------------------------------------------------------------------
 * Benchmark Query:
 *-----------------------------------------------------------------------------
 */

bench :- a(_,_), fail.
bench.


/*-----------------------------------------------------------------------------
 * Benchmark Query with Writing of Result:
 *-----------------------------------------------------------------------------
 */

bench_write(Stream) :-
	a(X,Y),
	write(Stream, X),
	write(Stream, '\t'),
	write(Stream, Y),
	write(Stream, '\n'),
	fail.
bench_write(_).


/*-----------------------------------------------------------------------------
 * The rest is the same for all XSB benchmarks:
 *-----------------------------------------------------------------------------
 */

:- consult('bench_lib.P').

